/*
 * iat.h - in-process Import Address Table function hooking
 * Copyright (c) 2005-2006, D.C. van Moolenbroek, XISE <dc@xise.nl>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _IAT_H
#define _IAT_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
  char *name;  // name of the function (NULL if none)
  int ordinal; // ordinal of the function (negative if none)
  void *proc;  // address of procedure to call instead
  void *next;  // next procedure in the hook chain (set to NULL initially)
  void *ptr;   // pointer into the import table (set to NULL initially)
} iat_hooks;

/*
 * function:    iat_hook
 * purpose:     replace addresses of function calls in a certain module's
 *              import address table (IAT) for a given imported DLL
 * parameters:  module    the handle of the module to change the IAT for;
 *                        for the base executable, use GetModuleHandle(NULL)
 *              import    name of the DLL that this module has loaded, for
 *                        which function pointers have to be replaced
 *              hooks     pointer to an array of IAT hooks, where for each the
 *                        first three items have to be filled in accordingly
 *              nhooks    number of entries in the hooks array
 * returns:     the number of hooks successfully placed
 */
extern int iat_hook(HINSTANCE module, char *import, iat_hooks *hooks,
 int nhooks);

/*
 * function:    iat_unhook
 * purpose:     remove hooks placed previously with iat_hook
 * parameters:  hooks     pointer to an array of IAT hooks
 *              nhooks    number of entries in the hooks array
 * returns:     the number of hooks successfully restored
 */
extern int iat_unhook(iat_hooks *hooks, int nhooks);

#ifdef __cplusplus
}
#endif

#endif /*_IAT_H*/
